package hdm.mi.sthbackend.controller;


import hdm.mi.sthbackend.dto.MatchDTO;
import hdm.mi.sthbackend.dto.PlayerDTO;
import hdm.mi.sthbackend.dto.TeamDTO;
import hdm.mi.sthbackend.dto.TournamentDTO;
import hdm.mi.sthbackend.exeptions.*;
import hdm.mi.sthbackend.model.Match;
import hdm.mi.sthbackend.service.TournamentService;
import hdm.mi.sthbackend.types.TeamMatchScore;
import hdm.mi.sthbackend.types.TeamName;
import hdm.mi.sthbackend.types.TournamentName;
import lombok.AllArgsConstructor;
import org.springframework.web.bind.annotation.*;

import java.util.UUID;

@RestController
@RequestMapping("/api/v1")
@AllArgsConstructor
public class TournamentController {

    private final TournamentService service;

    /**
     * Player Endpoints
     **/
    @GetMapping("players/{playerId}")
    public PlayerDTO findPlayerById(@PathVariable UUID playerId) throws PlayerIdNotFoundException {
        return service.findPlayerById(playerId);
    }

    @PatchMapping("/players/{playerId}")
    public UUID updatePlayerName(@PathVariable UUID playerId,
                                 @RequestBody String newPlayerName) throws PlayerIdNotFoundException {
        return service.updatePlayerName(playerId, newPlayerName);
    }

    /**
     * Team Endpoints
     **/
    @GetMapping("/teams/{teamId}")
    public TeamDTO findTeamById(@PathVariable UUID teamId) throws TeamIdNotFoundException {
        return service.findTeamById(teamId);
    }

    @PostMapping("/tournaments/{tournamentId}/teams")
    public TeamDTO createTeam(@PathVariable UUID tournamentId,
                              @RequestBody TeamName teamName) throws TournamentIdNotFoundException {
        return service.createAndAddTeamToTournament(tournamentId, teamName.getName());
    }

    @DeleteMapping("/tournaments/{tournamentId}/teams/{teamId}")
    public UUID removeTeamFromTournament(@PathVariable UUID tournamentId,
                                         @PathVariable UUID teamId) throws TournamentIdNotFoundException, TeamIdNotFoundException {
        return service.deleteAndRemoveTeamFromTournament(tournamentId, teamId);
    }

    @PostMapping("/teams/{teamId}/addPlayer")
    public PlayerDTO addPlayerToTeam(@PathVariable UUID teamId,
                                     @RequestBody String playerName) throws Exception {
        return service.addPlayerToTeam(teamId, playerName);
    }

    @DeleteMapping("/teams/{teamId}/players/{playerId}")
    public PlayerDTO removePlayerFromTeam(@PathVariable UUID teamId,
                                          @PathVariable UUID playerId) throws PlayerIdNotFoundException, TeamIdNotFoundException {
        return service.removePlayerFromTeam(teamId, playerId);
    }

    @PatchMapping("/teams/{teamId}/updateTeamName")
    public TeamDTO updateTeamName(@PathVariable UUID teamId, @RequestBody TeamName newTeamName) throws TeamIdNotFoundException {
        return service.updateTeamName(teamId, newTeamName.getName());
    }



    /**
     * Match Endpoints
     */
    @PatchMapping("matches/{matchId}/teams/{teamId}/assignTeamToMatch")
    public MatchDTO assignTeamToMatch(@PathVariable UUID matchId, @PathVariable UUID teamId, @RequestBody TeamMatchScore score) throws MatchIdNotFoundException {
        return service.assignTeamToMatch(matchId, teamId, score.getScore());
    }

    @PatchMapping("matches/{matchId}/teams/{teamId}/updateScore")
    public MatchDTO updateScore(@PathVariable UUID matchId, @PathVariable UUID teamId, @RequestBody TeamMatchScore newScore) throws MatchIdNotFoundException, TeamIdNotFoundException {
        return service.updateTeamScore(matchId, teamId, newScore.getScore());
    }

    @PatchMapping("/match/{matchId}/determineWinner")
    public UUID determineWinner(@PathVariable UUID matchId) throws MatchIdNotFoundException, WinnerNotDeterminedException {
        return service.determineWinner(matchId);
    }


    /**
     * Tournament Endpoints
     */
    @GetMapping("/tournaments/{tournamentId}")
    public TournamentDTO findTournamentById(@PathVariable UUID tournamentId) throws TournamentIdNotFoundException {
        return service.getTournament(tournamentId);
    }

    @PostMapping("/tournaments")
    public TournamentDTO createTournament(@RequestBody TournamentName params) {
        return service.createTournament(params.getName());
    }

    @DeleteMapping("/tournaments/{tournamentId}")
    public TournamentDTO deleteTournament(@PathVariable UUID tournamentId) throws TournamentIdNotFoundException {
        return service.deleteTournament(tournamentId);
    }

    @PatchMapping("/tournaments/{tournamentId}/updateName")
    public UUID updateTournamentName(@PathVariable UUID tournamentId, @RequestBody String newTournamentName) throws TournamentIdNotFoundException {
        return service.updateTournamentName(tournamentId, newTournamentName);
    }

}
